#
#  sundtek control center
#  coded by giro77
#  support: http://www.i-have-a-dreambox.com
#
#
#  This plugin is licensed under the Creative Commons 
#  Attribution-NonCommercial-ShareAlike 3.0 Unported License.
#  To view a copy of this license, please visit
#  http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative
#  Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
#
#  Alternatively, this plugin may be distributed 
#  with devices from sundtek ltd. or sundtek germany.
#
#
#  Sundtek Control Center Plugin is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#

from Components.ActionMap import ActionMap
from Components.config import config, getConfigListEntry, ConfigSubsection, ConfigInteger, ConfigYesNo, ConfigText, ConfigSelection, configfile
from Components.ConfigList import ConfigListScreen
from Components.Input import Input
from Components.Label import Label
from Components.MenuList import MenuList
from Components.MultiContent import MultiContentEntryText
from Components.NimManager import nimmanager
from Components.Sources.StaticText import StaticText
from enigma import eConsoleAppContainer, getEnigmaVersionString, getDesktop
from Plugins.Plugin import PluginDescriptor
from Screens.ChoiceBox import ChoiceBox
from Screens.VirtualKeyBoard import VirtualKeyBoard
from Screens.Console import Console
from Screens.InputBox import InputBox
from Screens.Screen import Screen
from Screens.Standby import TryQuitMainloop
from Screens.MessageBox import MessageBox
from Tools.NumericalTextInput import NumericalTextInput
import os, re, time, datetime
import urllib
import array, struct, fcntl
from threading import Thread
from socket import socket, AF_INET, SOCK_STREAM, SOCK_DGRAM, SHUT_RDWR
SIOCGIFCONF = 0x8912
BYTES = 4096

# for localized texts
from . import _

## configs ################################################################

config.plugins.SundtekControlCenter = ConfigSubsection()
config.plugins.SundtekControlCenter.display = ConfigSelection(default="3", choices = [("0", _("nowhere")),("1", _("ext menu")),("2", _("scan menu")),("3", _("ext/scan menu"))])
config.plugins.SundtekControlCenter.dvbtransmission = ConfigSelection(default="0", choices = [("0", ("DVB-S/S2")),("1", ("DVB-C")),("2", ("DVB-T")),("3", ("DVB-T2"))])
config.plugins.SundtekControlCenter.secondstick = ConfigYesNo(default=False)
config.plugins.SundtekControlCenter.secdvbtransmission = ConfigSelection(default="0", choices = [("0", ("DVB-S/S2")),("1", ("DVB-C")),("2", ("DVB-T")),("3", ("DVB-T2"))])
config.plugins.SundtekControlCenter.thirdstick = ConfigYesNo(default=False)
config.plugins.SundtekControlCenter.thirddvbtransmission = ConfigSelection(default="0", choices = [("0", ("DVB-S/S2")),("1", ("DVB-C")),("2", ("DVB-T")),("3", ("DVB-T2"))])

config.plugins.SundtekControlCenter.sunconf = ConfigSubsection()
config.plugins.SundtekControlCenter.sunconf.support = ConfigSelection(default="0", choices = [("0", _("hide")),("1", _("show"))])
config.plugins.SundtekControlCenter.sunconf.autostart = ConfigYesNo(default=False)
config.plugins.SundtekControlCenter.sunconf.autoupdate = ConfigYesNo(default=False)
config.plugins.SundtekControlCenter.sunconf.vtuneracceleration = ConfigYesNo(default=False)
config.plugins.SundtekControlCenter.sunconf.searchversion = ConfigYesNo(default=True)
config.plugins.SundtekControlCenter.sunconf.loglevel = ConfigSelection(default="0", choices = [("0", _("off")),("1", _("min")),("2", _("max"))])
config.plugins.SundtekControlCenter.sunconf.dmhwpidfilter = ConfigSelection(default="0", choices = [("0", _("off")),("1", _("on"))])
config.plugins.SundtekControlCenter.sunconf.networkmode = ConfigSelection(default="0", choices = [("0", _("off")),("1", _("on"))])

config.plugins.SundtekControlCenter.usbnet = ConfigSubsection()
config.plugins.SundtekControlCenter.usbnet.selection = ConfigSelection(default="0", choices = [("0", _("via USB")),("1", _("via Network"))])
config.plugins.SundtekControlCenter.usbnet.networkip = ConfigText(default="0.0.0.0", visible_width = 50, fixed_size = False)

config.plugins.SundtekControlCenter.secusbnet = ConfigSubsection()
config.plugins.SundtekControlCenter.secusbnet.selection = ConfigSelection(default="0", choices = [("0", _("via USB")),("1", _("via Network"))])
config.plugins.SundtekControlCenter.secusbnet.networkip = ConfigText(default="0.0.0.0", visible_width = 50, fixed_size = False)

config.plugins.SundtekControlCenter.thirdusbnet = ConfigSubsection()
config.plugins.SundtekControlCenter.thirdusbnet.selection = ConfigSelection(default="0", choices = [("0", _("via USB")),("1", _("via Network"))])
config.plugins.SundtekControlCenter.thirdusbnet.networkip = ConfigText(default="0.0.0.0", visible_width = 50, fixed_size = False)

## version string #########################################################

sundtekcontrolcenter_version = "1.4.1-20150531-r0"
testOK = None

###########################################################################

class SundtekControlCenter(Screen, ConfigListScreen):
    nims = nimmanager.nimList() # get nim_sockets
    result = []
    for item in nims:
        if _('Sundtek') in item and _('DVB-') in item:
             result.append((item))
    imageone =""
    imagetwo =""
    framewidth = getDesktop(0).size().width()
    if framewidth >= 1024:
        if len(result) == 1:
            if ( _("DVB-C") in result[0]) or ( _("DVB-T2") in result[0]) or ( _("DVB-T2") in result[0]):
                imageone="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/images/sundtek_dvbc.png"
            else:
                imageone="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/images/sundtek_dvbs.png"
            skin = "<screen title=\"SundtekControlCenter\" position=\"center,100\" size=\"750,550\" name=\"SundtekControlCenter\">\
                    <ePixmap pixmap=\"/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/images/bg.png\" position=\"0,0\" size=\"750,124\" alphatest=\"on\"/>\
                    <ePixmap pixmap=\"skin_default/buttons/red.png\" position=\"75,385\" size=\"140,40\" alphatest=\"on\" />\
                    <ePixmap pixmap=\"skin_default/buttons/green.png\" position=\"225,385\" size=\"140,40\" alphatest=\"on\" />\
                    <ePixmap pixmap=\"skin_default/buttons/yellow.png\" position=\"375,385\" size=\"140,40\" alphatest=\"on\" />\
                    <ePixmap pixmap=\"skin_default/buttons/blue.png\" position=\"525,385\" size=\"140,40\" alphatest=\"on\" />\
                    <widget name=\"btt_red\" position=\"75,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;17\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"btt_green\" position=\"225,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;17\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"btt_yellow\" position=\"375,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;17\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"btt_blue\" position=\"525,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;17\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" /> \
                    <widget name=\"ok\" position=\"25,430\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"menu\" position=\"25,450\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"infos\" position=\"25,470\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"bouquets\" position=\"25,490\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"netservers\" position=\"25,510\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"driverupdateavail\" position=\"375,37\" zPosition=\"1\" size=\"350,30\" font=\"Regular;16\" halign=\"right\" valign=\"center\" transparent=\"1\" shadowColor=\"black\" shadowOffset=\"-3,-3\" foregroundColor=\"#00ffffff\" />\
                    <widget name=\"updateavail\" position=\"375,55\" zPosition=\"1\" size=\"350,30\" font=\"Regular;16\" halign=\"right\" valign=\"center\" transparent=\"1\" shadowColor=\"black\" shadowOffset=\"-3,-3\" foregroundColor=\"#00ffffff\" />\
                    <widget name=\"version\" position=\"575,75\" zPosition=\"1\" size=\"150,30\" font=\"Regular;16\" halign=\"right\" valign=\"center\" transparent=\"1\" shadowColor=\"black\" shadowOffset=\"-3,-3\" />\
                    <widget name=\"config\" position=\"240,135\" size=\"480,250\" scrollbarMode=\"showOnDemand\" zPosition=\"1\"/>\
                    <ePixmap position=\"35,130\" size=\"96,58\" pixmap=\""+ imageone +"\" transparent=\"1\" alphatest=\"on\" />\
                    <widget name=\"tunerone\" position=\"25,190\" zPosition=\"1\" size=\"215,60\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <ePixmap position=\"625, 500\" size=\"100,40\" pixmap=\"/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/plugin.png\" transparent=\"1\" alphatest=\"on\" />\
                </screen>"
        elif len(result) >= 2:
            if ( _("DVB-C") in result[0]) or ( _("DVB-T2") in result[0]) or ( _("DVB-T2") in result[0]):
                imageone="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/images/sundtek_dvbc.png"
            else:
                imageone="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/images/sundtek_dvbs.png"
            if ( _("DVB-C") in result[1]) or ( _("DVB-T2") in result[1]) or ( _("DVB-T2") in result[1]):
                imagetwo="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/images/sundtek_dvbc.png"
            else:
                imagetwo="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/images/sundtek_dvbs.png"
            skin = "<screen title=\"SundtekControlCenter\" position=\"center,100\" size=\"750,550\" name=\"SundtekControlCenter\">\
                    <ePixmap pixmap=\"/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/images/bg.png\" position=\"0,0\" size=\"750,124\" alphatest=\"on\"/>\
                    <ePixmap pixmap=\"skin_default/buttons/red.png\" position=\"75,385\" size=\"140,40\" alphatest=\"on\" />\
                    <ePixmap pixmap=\"skin_default/buttons/green.png\" position=\"225,385\" size=\"140,40\" alphatest=\"on\" />\
                    <ePixmap pixmap=\"skin_default/buttons/yellow.png\" position=\"375,385\" size=\"140,40\" alphatest=\"on\" />\
                    <ePixmap pixmap=\"skin_default/buttons/blue.png\" position=\"525,385\" size=\"140,40\" alphatest=\"on\" />\
                    <widget name=\"btt_red\" position=\"75,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;17\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"btt_green\" position=\"225,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;17\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"btt_yellow\" position=\"375,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;17\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"btt_blue\" position=\"525,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;17\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" /> \
                    <widget name=\"ok\" position=\"25,430\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"menu\" position=\"25,450\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"infos\" position=\"25,470\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"bouquets\" position=\"25,490\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"netservers\" position=\"25,510\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <widget name=\"driverupdateavail\" position=\"375,37\" zPosition=\"1\" size=\"350,30\" font=\"Regular;16\" halign=\"right\" valign=\"center\" transparent=\"1\" shadowColor=\"black\" shadowOffset=\"-3,-3\" foregroundColor=\"#00ffffff\" />\
                    <widget name=\"updateavail\" position=\"375,55\" zPosition=\"1\" size=\"350,30\" font=\"Regular;16\" halign=\"right\" valign=\"center\" transparent=\"1\" shadowColor=\"black\" shadowOffset=\"-3,-3\" foregroundColor=\"#00ffffff\" />\
                    <widget name=\"version\" position=\"575,75\" zPosition=\"1\" size=\"150,30\" font=\"Regular;16\" halign=\"right\" valign=\"center\" transparent=\"1\" shadowColor=\"black\" shadowOffset=\"-3,-3\" />\
                    <widget name=\"config\" position=\"240,135\" size=\"480,250\" scrollbarMode=\"showOnDemand\" zPosition=\"1\"/>\
                    <ePixmap position=\"35,130\" size=\"96,58\" pixmap=\""+ imageone +"\" transparent=\"1\" alphatest=\"on\" />\
                    <widget name=\"tunerone\" position=\"25,190\" zPosition=\"1\" size=\"215,60\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <ePixmap position=\"35,255\" size=\"96,58\" pixmap=\""+ imagetwo +"\" transparent=\"1\" alphatest=\"on\" />\
                    <widget name=\"tunertwo\" position=\"25,315\" zPosition=\"1\" size=\"215,68\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                    <ePixmap position=\"625, 500\" size=\"100,40\" pixmap=\"/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/plugin.png\" transparent=\"1\" alphatest=\"on\" />\
                </screen>"
        else:
            skin = """
                <screen title="SundtekControlCenter" position="100,100" size="750,550" name="SundtekControlCenter">
                    <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/images/bg.png" position="0,0" size="750,124" alphatest="on"/>
                    <ePixmap pixmap="skin_default/buttons/red.png" position="75,385" size="140,40" alphatest="on" />
                    <ePixmap pixmap="skin_default/buttons/green.png" position="225,385" size="140,40" alphatest="on" />
                    <ePixmap pixmap="skin_default/buttons/yellow.png" position="375,385" size="140,40" alphatest="on" />
                    <ePixmap pixmap="skin_default/buttons/blue.png" position="525,385" size="140,40" alphatest="on" />
                    <widget name="btt_red" position="75,385" zPosition="1" size="140,40" font="Regular;17" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="btt_green" position="225,385" zPosition="1" size="140,40" font="Regular;17" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="btt_yellow" position="375,385" zPosition="1" size="140,40" font="Regular;17" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="btt_blue" position="525,385" zPosition="1" size="140,40" font="Regular;17" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" /> 
                    <widget name="ok" position="25,430" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="menu" position="25,450" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="infos" position="25,470" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="bouquets" position="25,490" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="netservers" position="25,510" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="driverupdateavail" position="375,37" zPosition="1" size="350,30" font="Regular;16" halign="right" valign="center" transparent="1" shadowColor="black" shadowOffset="-3,-3" foregroundColor="#00ffffff" />
                    <widget name="updateavail" position="375,55" zPosition="1" size="350,30" font="Regular;16" halign="right" valign="center" transparent="1" shadowColor="black" shadowOffset="-3,-3" foregroundColor="#00ffffff" />
                    <widget name="version" position="575,75" zPosition="1" size="150,30" font="Regular;16" halign="right" valign="center" transparent="1" shadowColor="black" shadowOffset="-3,-3" />
                    <widget name="config" position="240,135" size="480,245" scrollbarMode="showOnDemand" zPosition="1"/>
                    <widget name="tunerone" position="25,190" zPosition="1" size="215,60" font="Regular;16" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <ePixmap position="625, 500" size="100,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/plugin.png" transparent="1" alphatest="on" />
                </screen>"""
    else:
        skin = """
                <screen title="SundtekControlCenter" position="40,60" size="620,480" name="SundtekControlCenter">
                    <ePixmap pixmap="skin_default/buttons/red.png" position="10,320" size="140,40" alphatest="on" />
                    <ePixmap pixmap="skin_default/buttons/green.png" position="160,320" size="140,40" alphatest="on" />
                    <ePixmap pixmap="skin_default/buttons/yellow.png" position="310,320" size="140,40" alphatest="on" />
                    <ePixmap pixmap="skin_default/buttons/blue.png" position="460,320" size="140,40" alphatest="on" />
                    <widget name="btt_red" position="10,320" zPosition="1" size="140,40" font="Regular;17" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="btt_green" position="160,320" zPosition="1" size="140,40" font="Regular;17" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="btt_yellow" position="310,320" zPosition="1" size="140,40" font="Regular;17" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="btt_blue" position="460,320" zPosition="1" size="140,40" font="Regular;17" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" /> 
                    <widget name="ok" position="25,370" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="menu" position="25,390" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="infos" position="25,410" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="bouquets" position="25,430" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="netservers" position="25,450" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="driverupdateavail" position="245,30" zPosition="1" size="350,30" font="Regular;16" halign="right" valign="center" transparent="1" shadowColor="black" shadowOffset="-3,-3" foregroundColor="#00ffffff" />
                    <widget name="updateavail" position="245,50" zPosition="1" size="350,30" font="Regular;16" halign="right" valign="center" transparent="1" shadowColor="black" shadowOffset="-3,-3" foregroundColor="#00ffffff" />
                    <widget name="version" position="445,10" zPosition="1" size="150,30" font="Regular;16" halign="right" valign="center" transparent="1" shadowColor="black" shadowOffset="-3,-3" />
                    <widget name="config" position="25,100" size="570,200" scrollbarMode="showOnDemand" zPosition="1"/>
                    <widget name="tunerone" position="25,10" zPosition="1" size="215,60" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <widget name="tunertwo" position="220,10" zPosition="1" size="215,68" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                    <ePixmap position="495, 410" size="100,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/plugin.png" transparent="1" alphatest="on" />
                </screen>"""

    def __init__(self, session, args=0):
        Screen.__init__(self, session)
        ConfigListScreen.__init__(self, [])
        ### get nim_socket informations
        result = []
        i=0
        nims = nimmanager.nimList()
        for item in nims:
            if _('Sundtek') in item and _('DVB-') in item:
                 result.append((item))
                 i+=1
        self.total = i
        self.network = False
        self.updateSettingList()
        self["btt_red"] = Label(_("Exit"))
        self["btt_green"] = Label(_("Save setup"))
        self["btt_yellow"] = Label(_("Restart"))
        self["btt_blue"] = Label(_("Save and start tuner"))
        self["ok"] = Label(_("OK/ green = activate settings"))
        self["menu"] = Label(_("Menu = call further functions..."))
        self["infos"] = Label(_("Info = show tuner informations"))
        self["bouquets"] = Label(_("Bouquet + = install or update driver"))
        self["netservers"] = Label(_("Bouquet - = scan for IPTV server addresses"))
        self["version"] = Label(sundtekcontrolcenter_version)
        self["tunerone"] = Label("")
        self["tunertwo"] = Label("")
        self["updateavail"] = Label("")
        if os.path.exists("/opt/bin/mediaclient"):
            self["driverupdateavail"] = Label("")
        else:
            self["driverupdateavail"] = Label("n/a")

        ### get tunerinformations
        if len(result) == 1:
            self["tunerone"] = Label(str(result[0]))
        elif len(result) >= 2:
            self["tunerone"] = Label(str(result[0]))
            more_tuners = str(result[1])
            if len(result) > 2:
                more_tuners += "\n" + str(result[2])
            self["tunertwo"] = Label(more_tuners)
        else:
            self["tunerone"] = Label(_("No stick found"))
        self["actions"] = ActionMap(["MenuActions", "OkCancelActions", "ChannelSelectBaseActions", "ColorActions","ChannelSelectEPGActions"], 
        {
            "menu": self.menu,
            "ok": self.save,
            "cancel": self.cancel,
            "red": self.cancel,
            "green": self.save,
            "yellow": self.restartwhat,
            "blue": self.tunerstart,
            "showEPGList": self.dvbinfo,
            "nextBouquet": self.checkdriverversion,
            "prevBouquet": self.scannetwork,
        },-2)
        self.onLayoutFinish.append(self.layoutFinished)

    ########################################################

    def afterNetworkTest(self, net=True):
		### search for sundtekcontrolcenter updates   
		try: 
			version = urllib.urlopen('http://shockstyle.de/sundtek/version.html').read()
		except:
			version = "n/a"
		if version == sundtekcontrolcenter_version:
			self["updateavail"].setText("")
		elif version == "n/a":
			self["updateavail"].setText("n/a")
		else:
			try:
				current_version = int(sundtekcontrolcenter_version[6:14])
				available_version = int(version[6:14])
				if available_version > current_version:
					self["updateavail"].setText(_("scc update available"))
			except:
				self["updateavail"].setText(_("scc update available"))
		### search for sundtek driver update
		s = r"(?P<year>\d{2})(?P<month>\d{2})(?P<day>\d{2}).(?P<hours>\d{2})(?P<minutes>\d{2})(?P<seconds>\d{2})"
		pattern = re.compile(s)
		try:
			netdriver = urllib.urlopen('http://sundtek.de/media/latest.phtml?scc').read()
		except:
			netdriver = "n/a"
		match = pattern.search(netdriver)
		if match:
			netdriver = match.group("year")+match.group("month")+match.group("day")+match.group("hours")+match.group("minutes")+match.group("seconds")
		else:
			netdriver = "n/a"
		s = r"(?P<year>\d{2})-(?P<month>\d{2})-(?P<day>\d{2}) (?P<hours>\d{2}):(?P<minutes>\d{2}):(?P<seconds>\d{2})"
		pattern = re.compile(s)
		if os.path.exists("/opt/bin/mediaclient"):
			installeddriver = os.popen("/opt/bin/mediaclient --build", "r").read()
		else:
			installeddriver ="n/a"
		match = pattern.search(installeddriver)
		if match:
			installeddriver = match.group("year")+match.group("month")+match.group("day")+match.group("hours")+match.group("minutes")+match.group("seconds")
		else:
			installeddriver = "n/a"
		if (netdriver != "n/a") and (installeddriver != "n/a"):
			if int(netdriver) <= int(installeddriver):
				self["driverupdateavail"].setText("")
			else:
				self["driverupdateavail"].setText(_("driver update available"))
		else:
			self["driverupdateavail"].setText("n/a")

    def keyLeft(self):
		ConfigListScreen.keyLeft(self)
		self.updateSettingList()
		self.virtualKeyboard()

    def keyRight(self):
		ConfigListScreen.keyRight(self)
		self.updateSettingList()
		self.virtualKeyboard()

    def virtualKeyboard(self):
		idx = self["config"].getCurrent()
		if idx:
			if idx[1] == config.plugins.SundtekControlCenter.usbnet.networkip:
				self.openVK(config.plugins.SundtekControlCenter.usbnet.networkip.value)
			elif idx[1] == config.plugins.SundtekControlCenter.secusbnet.networkip:
				self.openVK(config.plugins.SundtekControlCenter.secusbnet.networkip.value)
			elif idx[1] == config.plugins.SundtekControlCenter.thirdusbnet.networkip:
				self.openVK(config.plugins.SundtekControlCenter.thirdusbnet.networkip.value)

    def openVK(self, current_config=None):
		if current_config is not None:
			title_text = _("Please enter IP:")
			old_text = current_config
		self.session.openWithCallback(self.VKCallback, VirtualKeyBoard, title=title_text, text=old_text)

    def VKCallback(self, answer):
		if answer:
			cur = self["config"].getCurrent()
			if cur:
				if cur[1] == config.plugins.SundtekControlCenter.usbnet.networkip:
					config.plugins.SundtekControlCenter.usbnet.networkip.value = answer
				elif cur[1] == config.plugins.SundtekControlCenter.secusbnet.networkip:
					config.plugins.SundtekControlCenter.secusbnet.networkip.value = answer
				elif cur[1] == config.plugins.SundtekControlCenter.thirdusbnet.networkip:
					config.plugins.SundtekControlCenter.thirdusbnet.networkip.value = answer
				self.updateSettingList()

    def save(self):
		for x in self["config"].list:
			x[1].save()
		configfile.save()
		self.sundtekconfigfile()
		if self.setsettings(True):
			self.close(True, self.session)

    def cancel(self):
        for x in self["config"].list:
            x[1].cancel()
        self.close(False, self.session)

    def menu(self):
        options = [
            (_("Install or Update sundtek Driver"), self.checkdriverversion),
            (_("Scan for IPTV server addresses"), self.scannetwork),
            (_("Enable / Disable IP server"), self.enablenetwork),
            (_("Service Mode"), self.enableservicemode),
            (_("Show tuner informations"), self.dvbinfo),
            (_("Backup Drivers to HDD"), self.driverbackup),
            (_("Restore Drivers from HDD"), self.driverrestore),
            (_("Check for a newer version of the SundtekControlCenter"), self.selfupdate), 
        ]
        if os.path.exists("/etc/sundtek.conf"):
            options.append((_("Show /etc/sundtek.conf"), self.configinfo))
        self.session.openWithCallback(self.thismenuCallback, ChoiceBox, list = options)

    def configinfo(self):
		self.prompt("cat /etc/sundtek.conf")

    def thismenuCallback(self,ret):
        ret and ret[1]()

    def updateSettingList(self):
        list = [] ### creating list
        list.append(getConfigListEntry(_("* Configuration support"), config.plugins.SundtekControlCenter.sunconf.support))
        if config.plugins.SundtekControlCenter.sunconf.support.value == "1":
            sublist = [
                getConfigListEntry(_("      Autostart"), config.plugins.SundtekControlCenter.sunconf.autostart),
                getConfigListEntry(_("      Driver autoupdate"), config.plugins.SundtekControlCenter.sunconf.autoupdate),
                getConfigListEntry(_("      Loglevel"), config.plugins.SundtekControlCenter.sunconf.loglevel),
                getConfigListEntry(_("      Networkmode"), config.plugins.SundtekControlCenter.sunconf.networkmode),
                getConfigListEntry(_("      dmhwpidfilter"), config.plugins.SundtekControlCenter.sunconf.dmhwpidfilter),
                getConfigListEntry(_("      vTuner acceleration for DM800"), config.plugins.SundtekControlCenter.sunconf.vtuneracceleration),
            ]
            list.extend(sublist)
        list.append(getConfigListEntry(_("DVB Transmission Way"), config.plugins.SundtekControlCenter.dvbtransmission)) 
        list.append(getConfigListEntry(_("* USB/Network"), config.plugins.SundtekControlCenter.usbnet.selection)) 
        if config.plugins.SundtekControlCenter.usbnet.selection.value == "1": ## if networking then add ip mask to list
            sublist = [
                getConfigListEntry(_("      Network IP"), config.plugins.SundtekControlCenter.usbnet.networkip)
            ]
            list.extend(sublist)
        list.append(getConfigListEntry(_("* Use Second Stick"), config.plugins.SundtekControlCenter.secondstick))
        if config.plugins.SundtekControlCenter.secondstick.value: 
            list.append(getConfigListEntry(_("      DVB Transmission Way, Stick 2"), config.plugins.SundtekControlCenter.secdvbtransmission)) 
            list.append(getConfigListEntry(_("*     USB/Network"), config.plugins.SundtekControlCenter.secusbnet.selection))
            
            if config.plugins.SundtekControlCenter.secusbnet.selection.value == "1": ## if networking then add ip mask to list
                secsublist = [
                    getConfigListEntry(_("          Network IP"), config.plugins.SundtekControlCenter.secusbnet.networkip)
                ]
                list.extend(secsublist)
        else: 
            config.plugins.SundtekControlCenter.secusbnet.selection.value = "0"
        list.append(getConfigListEntry(_("* Use Third Stick"), config.plugins.SundtekControlCenter.thirdstick))
        if config.plugins.SundtekControlCenter.thirdstick.value: 
            list.append(getConfigListEntry(_("      DVB Transmission Way, Stick 3"), config.plugins.SundtekControlCenter.thirddvbtransmission)) 
            list.append(getConfigListEntry(_("*     USB/Network"), config.plugins.SundtekControlCenter.thirdusbnet.selection))
            if config.plugins.SundtekControlCenter.thirdusbnet.selection.value == "1": ## if networking then add ip mask to list
                thirdsublist = [
                    getConfigListEntry(_("          Network IP"), config.plugins.SundtekControlCenter.thirdusbnet.networkip)
                ]
                list.extend(thirdsublist)
        else: 
            config.plugins.SundtekControlCenter.thirdusbnet.selection.value = "0"
        list.append(getConfigListEntry(_("Auto checking new version driver/plugin"), config.plugins.SundtekControlCenter.sunconf.searchversion))
        list.append(getConfigListEntry(_("Add show plugin"), config.plugins.SundtekControlCenter.display))
        self["config"].list = list
        self["config"].l.setList(list)

    def layoutFinished(self):
		self.setTitle(_("Sundtek Control Center"))
		if config.plugins.SundtekControlCenter.sunconf.searchversion.value:
			self.startTestNetwork()
		else:
			self.network = True

    def startTestNetwork(self):
		self.testThread = Thread(target=self.start_test)
		self.testThread.start()

    def get_iface_list(self):
		names = array.array('B', '\0' * BYTES)
		sck = socket(AF_INET, SOCK_DGRAM)
		bytelen = struct.unpack('iL', fcntl.ioctl(sck.fileno(), SIOCGIFCONF, struct.pack('iL', BYTES, names.buffer_info()[0])))[0]
		sck.close()
		namestr = names.tostring()
		return [namestr[i:i+32].split('\0', 1)[0] for i in range(0, bytelen, 32)]

    def start_test(self):
		global testOK
		link = "down"
		for iface in self.get_iface_list():
			if "lo" in iface: continue
			if os.path.exists("/sys/class/net/%s/operstate"%(iface)):
				fd = open("/sys/class/net/%s/operstate"%(iface), "r")
				link = fd.read().strip()
				fd.close()
			if link != "down": break
		if link != "down":
			s = socket(AF_INET, SOCK_STREAM)
			s.settimeout(2.0)
			try:
				testOK = not bool(s.connect_ex(("www.sundtek.de", 80)))
			except:
				testOK = None
			if not testOK:
				self["updateavail"].setText(_("www.sundtek.de unavailable"))
			else:
				s.shutdown(SHUT_RDWR)
				self.network = True
				self.afterNetworkTest()
			s.close()
		else:
			self["updateavail"].setText(_("Not found network connection"))

    #### scc update
    def selfupdate(self):
		if not self.network: return
		try: 
			self.version = urllib.urlopen('http://shockstyle.de/sundtek/version.html').read()
		except:
			self.version = "n/a"
		if self.version == sundtekcontrolcenter_version:
			self.session.open(MessageBox, _("Latest version is already installed"), MessageBox.TYPE_INFO, 7)
		elif self.version == "n/a":
			self.session.open(MessageBox, _("Check not possible. Please ensure that the Box is connected to the internet."), MessageBox.TYPE_INFO, 7)
		else:
			update = False
			try:
				current_version = int(sundtekcontrolcenter_version[6:14])
				available_version = int(self.version[6:14])
				if available_version > current_version:
					update = True
				else:
					self.session.open(MessageBox, _("Latest version is already installed"), MessageBox.TYPE_INFO, 7)
			except:
				update = True
			if update:
				self.session.openWithCallback(self.updateplugin, MessageBox, _("A newer version of the sundtekcontrolcenter plugin is available. Install now?"), MessageBox.TYPE_YESNO)

    def updateplugin(self, result):
        if result:
            self.session.open(MessageBox, _("The SundtekControlCenter is updated in background"), MessageBox.TYPE_INFO, 7)
            self.container=eConsoleAppContainer()
            self.container.appClosed.append(self.finishedSelfUpdate)
            self.container.execute("opkg install --force-overwrite " + "http://shockstyle.de/sundtek/enigma2-plugin-extensions-sundtekcontrolcenter_"+str(self.version)+"_all.ipk")

    def finishedSelfUpdate(self, result):
        self.session.openWithCallback(self.restartbox, MessageBox, _("SundtekControlCenter successfully updated! You need to restart Enigma2. Restart now?"), MessageBox.TYPE_YESNO) 

    def restartbox(self, result):
        if result:
            self.session.open(TryQuitMainloop, 3)

    #### check sundtek driverversion
    def checkdriverversion(self):
		if not self.network: return
		s = r"(?P<year>\d{2})(?P<month>\d{2})(?P<day>\d{2}).(?P<hours>\d{2})(?P<minutes>\d{2})(?P<seconds>\d{2})"
		pattern = re.compile(s)
		text = _("Build date :")
		try:
			netdriver = urllib.urlopen('http://sundtek.de/media/latest.phtml?scc').read() 
		except:
			netdriver = text +"n/a" 
		match = pattern.search(netdriver)
		if match:
			match = match.group("year")+" "+match.group("month")+" "+match.group("day")+" "+match.group("hours")+" "+match.group("minutes")+" "+match.group("seconds")
			sundtekdriverdate = time.strptime(match, "%y %m %d %H %M %S")
			netdriver = text+time.strftime("%Y-%m-%d %H:%M:%S", sundtekdriverdate)
		else:
			netdriver = text + "n/a"
		if os.path.exists("/opt/bin/mediaclient"):
			installeddriver = os.popen("/opt/bin/mediaclient --build", "r").read()
			if installeddriver.startswith("Build date:"):
				try:
					installeddriver = text + installeddriver[11:]
				except:
					pass
		else:
			installeddriver = text + "n/a"
		self.session.openWithCallback(self.disclaimer, MessageBox, _("latest sundtek driver version:\n")+netdriver+"\n"+_("\nyour driver version:\n")+installeddriver+_("\nUpdate to current sundtek driver version?"), MessageBox.TYPE_YESNO)

    def disclaimer(self, result): 
        if result:
            os.popen("chmod 755 /usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/sundtekinstall.sh > /dev/null 2>&1")
            self.prompt("/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/sundtekinstall.sh")

    #### search for ip servers
    def parseforip(self):
		lines = os.popen("/opt/bin/mediaclient --scan-network").readlines()
		sticks = []
		if len(lines) == 6:
			sticks = [0]
		elif len(lines) >= 7: 
			sticks = [0,0]
		if len(sticks) == 1:
			sticks[0] = lines[4].split("|")
			sticks[0] = sticks[0][0].strip()+":"+sticks[0][1].strip()
		elif len(sticks) >= 2:
			sticks[0] = lines[4].split("|")
			sticks[1] = lines[5].split("|")
			sticks[0] = sticks[0][0].strip()+":"+sticks[0][1].strip()
			sticks[1] = sticks[1][0].strip()+":"+sticks[1][1].strip()
			try:
				sticks[2] = lines[6].split("|")
				sticks[2] = sticks[2][0].strip()+":"+sticks[2][1].strip()
			except:
				pass
		else:
			### no info found 
			sticks = []
		return sticks

    def scannetwork(self):
		if os.path.exists("/opt/bin/mediaclient"):
			if len(self.parseforip()) == 1:
				self.session.openWithCallback(self.usenetip, MessageBox, _("Use following address as media server?\n")+self.parseforip()[0], MessageBox.TYPE_YESNO)
			elif len(self.parseforip()) >= 2:
				try:
					third = "\n"+self.parseforip()[2]
				except:
					third = ''
				self.session.openWithCallback(self.usetwonetip, MessageBox, _("Use following addresses?\n")+self.parseforip()[0]+"\n"+self.parseforip()[1] + third, MessageBox.TYPE_YESNO)
			else:
				self.session.open(MessageBox, _("No IPTV media server found"), MessageBox.TYPE_INFO, 7)

    def usenetip(self, result): 
        if result:
            config.plugins.SundtekControlCenter.usbnet.selection.value = "1"
            config.plugins.SundtekControlCenter.usbnet.networkip.value = self.parseforip()[0]
            self.updateSettingList()

    def usetwonetip(self, result): 
		if result:
			config.plugins.SundtekControlCenter.usbnet.selection.value = "1"
			config.plugins.SundtekControlCenter.usbnet.networkip.value = self.parseforip()[0]
			config.plugins.SundtekControlCenter.secondstick.value = True
			config.plugins.SundtekControlCenter.secusbnet.selection.value = "1"
			config.plugins.SundtekControlCenter.secusbnet.networkip.value = self.parseforip()[1]
			try:
				value = self.parseforip()[2]
				if value is not None:
					config.plugins.SundtekControlCenter.thirdstick.value = True
					config.plugins.SundtekControlCenter.thirdusbnet.selection.value = "1"
					config.plugins.SundtekControlCenter.thirdusbnet.networkip.value = value
			except:
				pass
			self.updateSettingList()

    #### enable network
    def enablenetwork(self):
		if not os.path.exists("/opt/bin/mediaclient"):
			return
		options = [
			(_("Enable the IP server"), self.startipserver),
			(_("Disable the IP server"), self.stopipserver),
		]
		self.session.openWithCallback(self.thismenuCallback, ChoiceBox, list = options)

    def startipserver(self):
        os.popen("/opt/bin/mediaclient --enablenetwork=on")
        self.session.open(MessageBox, _("IP server is enabled"), MessageBox.TYPE_INFO)

    def stopipserver(self):
        os.popen("/opt/bin/mediaclient --enablenetwork=off")
        self.session.open(MessageBox, _("IP server is disabled"), MessageBox.TYPE_INFO)

    #### service mode
    def enableservicemode(self):
		if not os.path.exists("/opt/bin/mediaclient"):
			return
		self.session.openWithCallback(self.startservicemode, MessageBox, _("Service Mode starts a tunnel to the sundtek server. Please start service mode only if requested by sundtek support.\n\nStart Service Mode now?"), MessageBox.TYPE_YESNO)

    def startservicemode(self, result): 
        if result:
            os.popen("/opt/bin/mediaclient --portforward 23")
            self.prompt("/opt/bin/mediaclient --portforward 23")

    #### backup
    def driverbackup(self):
		try:
			mounts = open("/proc/mounts")
		except:
			return
		hdd = False
		mountcheck = mounts.readlines()
		mounts.close()
		for line in mountcheck:
			if '/media/hdd' in line:
				hdd = True
				break
		if not hdd:
			self.session.open(MessageBox, _("Not found folder '/media/hdd'."), MessageBox.TYPE_INFO, 5)
			return
		backuppath = "/media/hdd/backup/SundtekBackup"
		now = datetime.datetime.now()
		backupfile = "sundtek-"+now.strftime("%Y%m%d-%H%M")+".tar"
		if (not os.path.exists(backuppath)): # backup folder does not yet exist
			os.makedirs(backuppath)
		self.prompt("tar -czvf"+" "+backuppath+"/"+backupfile+" /usr/sundtek /opt/bin /etc/sundtek.conf /etc/sundtek.net") 

    #### restore
    def driverrestore(self):
        restorepath = "/media/hdd/backup/SundtekBackup"
        if (os.path.exists(restorepath)): # backup folder exists
            backupfile = [line.rstrip('\n') for line in (os.popen("ls "+restorepath+"/sundtek*tar").readlines())]
            if len(backupfile) > 0: # found at least one backup in the backupfolder
                options = []
                restorelist = []
                restorelist = [item.lstrip(restorepath) for item in backupfile]
                for item in backupfile:
                    options.append((_(restorelist.pop()), None))
                self.session.openWithCallback(self.restoremenuCallback, ChoiceBox, list = options)
            else: # es wurde kein backup file gefunden
                self.session.open(MessageBox, _("No backup file was found."), MessageBox.TYPE_INFO, 10)
        else: 
            self.session.open(MessageBox, _("No backups found."), MessageBox.TYPE_INFO, 10)

    def restoremenuCallback(self,ret):
        if ret:
            restore = ("tar -xvpzf /media/hdd/backup/SundtekBackup/%s -C /") % str(ret[0])
            self.prompt(restore) # restore from root
            self.session.open(MessageBox, _("Restoreing:  %s") % str(ret[0]), MessageBox.TYPE_INFO, 7)
        else:
            self.session.open(MessageBox, _("Restore was canceled"), MessageBox.TYPE_INFO, 7)

    #### help
    #def helper(self):
        #self.prompt("cat /usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/helpfile.txt")

#### yellow key - restart section
    def restartwhat(self):
        options = [
            (_("Restart E2"), self.restartenigma),
            (_("Reboot box"), self.restartingbox),
        ]
        self.session.openWithCallback(self.thismenuCallback, ChoiceBox, list = options)

    def restartenigma(self):
        for x in self["config"].list:
            x[1].save()
        configfile.save()
        self.sundtekconfigfile()
        self.setsettings()
        self.session.open(TryQuitMainloop, 3)

    def restartingbox(self):
        for x in self["config"].list:
            x[1].save()
        configfile.save()
        self.sundtekconfigfile()
        self.setsettings()
        self.session.open(TryQuitMainloop, 2)

    def dvbinfo(self):
		if os.path.exists("/usr/sundtek/sun_dvb.sh"):
			self.prompt("/usr/sundtek/sun_dvb.sh info")

    def installdriverrequest(self, result):
        if result:
            self.session.openWithCallback(self.disclaimer, MessageBox, _("Sundtek legal notice:\nThis software comes without any warranty, use it at your own risk?\nContinue?"), MessageBox.TYPE_YESNO)

    def prompt(self, com):
        self.session.open(Console,(""), ["%s" %com])

####################################################################

#### config file section
    def parsefornetworkstickserial(self):
		lines = os.popen("/opt/bin/mediaclient --scan-network").readlines()
		sticks = []
		if len(lines) == 6:
			sticks = [0]
		elif len(lines) >= 7: 
			sticks = [0,0]
		if len(sticks) == 1:
			sticks[0] = lines[4].split("|")
			sticks[0] = sticks[0][0].strip()+":"+sticks[0][1].strip()
		elif len(sticks) >= 2:
			sticks[0] = lines[4].split("|")
			sticks[1] = lines[5].split("|")
			sticks[0] = sticks[0][0].strip()+":"+sticks[0][1].strip()
			sticks[1] = sticks[1][0].strip()+":"+sticks[1][1].strip()
			try:
				sticks[2] = lines[6].split("|")
				sticks[2] = sticks[2][0].strip()+":"+sticks[2][1].strip()
			except:
				pass
		else:
			### no info found 
			sticks = []
		return sticks

    def sundtekconfigfile(self):
        conffile ="/etc/sundtek.conf"
        now = datetime.datetime.now()
        results = []
        if (config.plugins.SundtekControlCenter.usbnet.selection.value == "1"):
            #### network
            results = self.parsefornetworkstickserial()
        else:
            ### no network
            s = r"\[SERIAL\]:\n.+ID: (\w+)"
            pattern = re.compile(s)
            #devices = os.popen("cat /usr/sundtek/test.txt").read()
            devices = os.popen("/opt/bin/mediaclient -e").read()
            i = 0
            while True:
                match = pattern.search(devices,i)
                if match:
                    results.append(match.group(1))
                    i = match.end()+1
                else:
                    break
        #### header
        header1 = ("# sundtek configuration file - /etc/sundtek.conf\n")
        header2 = ("# created / modified: "+(now.strftime("%b %d %Y, %H:%Mh"))+" by sundtekcontrolcenter "+sundtekcontrolcenter_version+"\n\n")
        header = header1+header2

        #### configuration (loglevel, autoupdate, vtuneracceleration, dmhwpidfilter, networkmode)
        loglevel=""
        autoupdate=""
        vtuneracceleration=""
        dmhwpidfilter = ""
        networkmode = ""
        loglevel = str(config.plugins.SundtekControlCenter.sunconf.loglevel.value) # loglevel
        if (loglevel == "1"):
            os.system("/opt/bin/mediaclient --loglevel=min")
            loglevel = "loglevel = min\n"
        elif (loglevel == "2"):
            os.system("/opt/bin/mediaclient --loglevel=max")
            loglevel = "loglevel = max\n"
        else:
            os.system("/opt/bin/mediaclient --loglevel=off")
            loglevel = "loglevel = off\n"

        autoupdate = str(config.plugins.SundtekControlCenter.sunconf.autoupdate.value) # driver autoupdate
        if (autoupdate == "True"):
            autoupdate = "autoupdate = on\n"
        else:
            autoupdate = "autoupdate = off\n"
        dmhwpidfilter = str(config.plugins.SundtekControlCenter.sunconf.dmhwpidfilter.value) # dmhwpidfilter autoupdate
        if (dmhwpidfilter == "1"):
            dmhwpidfilter = "dmhwpidfilter = on\n"
        else:
            dmhwpidfilter = "dmhwpidfilter = off\n"
        vtuneracceleration = str(config.plugins.SundtekControlCenter.sunconf.vtuneracceleration.value) # vtuner acceleration
        if (vtuneracceleration == "True"):
            vtuneracceleration = "vtuner_acceleration = on\n"
        else:
            vtuneracceleration = "vtuner_acceleration = off\n"
        networkmode = str(config.plugins.SundtekControlCenter.sunconf.networkmode.value) #networkmode
        if (networkmode == "1"):
            networkmode = "enablenetwork = on\n"
        else:
            networkmode = "enablenetwork = off\n"

        #### sticks
        networkingip = ""
        secnetworkingip = ""
        thirdnetworkingip = ""
        netsection = ""
        net1l1 = ""
        net1l2 = ""
        net1l3 = ""
        if (config.plugins.SundtekControlCenter.usbnet.selection.value == "1"):
            networkingip = config.plugins.SundtekControlCenter.usbnet.networkip.value
            if networkingip == '':
                networkingip = config.plugins.SundtekControlCenter.usbnet.networkip.value = "0.0.0.0"
                config.plugins.SundtekControlCenter.usbnet.networkip.save()
            networkingip.strip()
            net1l1 = "###### configuration network\n[NETWORK]\ndevice="+networkingip+"\n"
        if config.plugins.SundtekControlCenter.secondstick.value:
            if config.plugins.SundtekControlCenter.secusbnet.selection.value == "1":
                secnetworkingip = config.plugins.SundtekControlCenter.secusbnet.networkip.value
                if secnetworkingip == '':
                    secnetworkingip = config.plugins.SundtekControlCenter.secusbnet.networkip.value = "0.0.0.0"
                    config.plugins.SundtekControlCenter.secusbnet.networkip.save()
                secnetworkingip.strip()
                if config.plugins.SundtekControlCenter.usbnet.selection.value == "1":
                    net1l2 = "device="+secnetworkingip+"\n"
                else:
                    net1l2 = "###### configuration network\n[NETWORK]\ndevice="+secnetworkingip+"\n"
        if config.plugins.SundtekControlCenter.thirdstick.value:
            if config.plugins.SundtekControlCenter.thirdusbnet.selection.value == "1":
                thirdnetworkingip = config.plugins.SundtekControlCenter.thirdusbnet.networkip.value
                if thirdnetworkingip == '':
                    thirdnetworkingip = config.plugins.SundtekControlCenter.thirdusbnet.networkip.value = "0.0.0.0"
                    config.plugins.SundtekControlCenter.thirdusbnet.networkip.save()
                thirdnetworkingip.strip()
                if config.plugins.SundtekControlCenter.usbnet.selection.value == "1" or config.plugins.SundtekControlCenter.secusbnet.selection.value == "1":
                    net1l3 = "device="+thirdnetworkingip+"\n"
                else:
                    net1l3 = "###### configuration network\n[NETWORK]\ndevice="+thirdnetworkingip+"\n"
        netsection = net1l1+net1l2+net1l3+"\n"
        stick1 = ""
        stick2 = ""
        stick3 = ""
        twotuners = False
        show_message = False

        #stick 1 data
        if len(results) >= 1:
            stick1l1 = ("###### configuration stick 1\n") 
            if (not config.plugins.SundtekControlCenter.usbnet.selection.value == "1"):
                stick1l2 = ("["+results.pop(0)+"]\n")
            elif config.plugins.SundtekControlCenter.usbnet.selection.value == "1":
                stick1l2 = "["+results.pop(0)+"]\n"+"netrecoverymode=on\n"
            dvbtransmission = str(config.plugins.SundtekControlCenter.dvbtransmission.value)
            if dvbtransmission == "1":
                stick1l3 = "initial_dvb_mode=DVBC\n"
            elif dvbtransmission == "2":
                stick1l3 = "initial_dvb_mode=DVBT\n"
            elif dvbtransmission == "3":
                stick1l3 = "initial_dvb_mode=DVBT2\n"   #might be DVBT2 here
            else:
                stick1l3 = ""
            stick1l4 = ("dreambox_support_fe1=on\n\n")
            stick1 = stick1l1+stick1l2+stick1l3+stick1l4
        if config.plugins.SundtekControlCenter.secondstick.value:
            if len(results) >= 1:
                twotuners = True
                #stick 2 data
                stick2l1 = ("###### configuration stick 2\n")
                if (not config.plugins.SundtekControlCenter.secusbnet.selection.value == "1"):
                    stick2l2 = ("["+results.pop(0)+"]\n")
                elif config.plugins.SundtekControlCenter.secusbnet.selection.value == "1":
                    stick2l2 = "["+results.pop(0)+"]\n"+"netrecoverymode=on\n"
                dvbtransmission = config.plugins.SundtekControlCenter.secdvbtransmission.value
                if dvbtransmission == "1":
                    stick2l3 = "initial_dvb_mode=DVBC\n"
                elif dvbtransmission == "2":
                    stick2l3 = "initial_dvb_mode=DVBT\n"
                elif dvbtransmission == "3":
                    stick2l3 = "initial_dvb_mode=DVBT2\n"
                else:
                    stick2l3 = ""
                stick2l4 = ("dreambox_support_fe1=on\n")
                stick2 = stick2l1+stick2l2+stick2l3+stick2l4
            else: # len < 2, only one or no ip found
                show_message = True
                self.session.open(MessageBox, _("Sorry, only one or no stick ID was found.\n\nPlease check that all sticks are plugged correctly. The config file was written without stick information."), MessageBox.TYPE_INFO, 15)
        if config.plugins.SundtekControlCenter.thirdstick.value:
            if (twotuners and len(results) >= 1) or (not twotuners and len(results) >= 2):
                #stick 3 data
                stick3l1 = ("###### configuration stick 3\n")
                if twotuners:
                    if (not config.plugins.SundtekControlCenter.thirdusbnet.selection.value == "1"):
                        stick3l2 = ("["+results.pop(0)+"]\n")
                    elif config.plugins.SundtekControlCenter.thirdusbnet.selection.value == "1":
                        stick3l2 = "["+results.pop(0)+"]\n"+"netrecoverymode=on\n"
                else:
                   if (not config.plugins.SundtekControlCenter.thirdusbnet.selection.value == "1"):
                        stick3l2 = ("["+results[1]+"]\n")
                   elif config.plugins.SundtekControlCenter.thirdusbnet.selection.value == "1":
                        stick3l2 = "["+results[1]+"]\n"+"netrecoverymode=on\n"
                dvbtransmission = config.plugins.SundtekControlCenter.thirddvbtransmission.value
                if dvbtransmission == "1":
                    stick3l3 = "initial_dvb_mode=DVBC\n"
                elif dvbtransmission == "2":
                    stick3l3 = "initial_dvb_mode=DVBT\n"
                elif dvbtransmission == "3":
                    stick3l3 = "initial_dvb_mode=DVBT2\n"
                else:
                    stick3l3 = ""
                stick3l4 = ("dreambox_support_fe1=on\n")
                stick3 = stick3l1+stick3l2+stick3l3+stick3l4
            else: # len < 2, only one or no ip found
                if not show_message:
                    self.session.open(MessageBox, _("Sorry, only one or no stick ID was found.\n\nPlease check that all sticks are plugged correctly. The config file was written without stick information."), MessageBox.TYPE_INFO, 15)
        data = header+loglevel+autoupdate+dmhwpidfilter+vtuneracceleration+networkmode+"\n"+netsection+stick1+stick2+stick3
        ### (over)write file
        f = open(conffile, "w")
        f.writelines(data)
        f.close()

####################################################################

#### settings 
    def setsettings(self, exit=False):
        ### check if driver is installed
        if ((not os.path.exists("/opt/bin/mediasrv")) or (not os.path.exists("/opt/bin/mediaclient")) or (not os.path.exists("/usr/sundtek/sun_dvb.sh"))):
            ## maybe driver not installed
            if (os.path.exists("/usr/sundtek/mediasrv") and os.path.exists("/usr/sundtek/mediaclient") and os.path.exists("/usr/sundtek/sun_dvb.sh")) and os.path.exists("/usr/sundtek/libmediaclient.so"):
                ## driver installed but no links in /opt/bin
                if (not os.path.exists("/opt/bin")):
                    os.makedirs("/opt/bin")
                if (not os.path.exists("/opt/lib")):
                    os.makedirs("/opt/lib")
                os.popen("ln -s /usr/sundtek/mediasrv /opt/bin/mediasrv")
                os.popen("ln -s /usr/sundtek/mediaclient /opt/bin/mediaclient")
                os.popen("ln -s /usr/sundtek/libmediaclient.so /opt/lib/libmediaclient.so")
            else:
                ## driver not installed
                self.session.openWithCallback(self.installdriverrequest, MessageBox, _("It seems the sundtek driver is not installed or not installed properly. Install the driver now?"), MessageBox.TYPE_YESNO)
                if exit: return False
        else:
            ### driver installed
            ### disable autostart
            if config.plugins.SundtekControlCenter.sunconf.autostart.value == False:
                self.prompt("/usr/sundtek/sun_dvb.sh noautostart")
                
            if config.plugins.SundtekControlCenter.usbnet.selection.value == "1":
                if config.plugins.SundtekControlCenter.sunconf.autostart.value == True:
                    if config.plugins.SundtekControlCenter.dvbtransmission.value == "0": #autostart DVBS
                        self.prompt("/usr/sundtek/sun_dvb.sh autostart_netdvb")
                    else:
                        self.prompt("/usr/sundtek/sun_dvb.sh autostart_net")
            else:
                if config.plugins.SundtekControlCenter.dvbtransmission.value == "1":
                    ### dvb-c
                    if config.plugins.SundtekControlCenter.sunconf.autostart.value == True:
                        ### enable autostart
                        self.prompt("/usr/sundtek/sun_dvb.sh autostart_c")
                elif config.plugins.SundtekControlCenter.dvbtransmission.value == "2":
                    ### dvb-t
                    if config.plugins.SundtekControlCenter.sunconf.autostart.value == True:
                        ### enable autostart
                        self.prompt("/usr/sundtek/sun_dvb.sh autostart_t")
                elif config.plugins.SundtekControlCenter.dvbtransmission.value == "3":
                    ### dvb-t2
                    if config.plugins.SundtekControlCenter.sunconf.autostart.value == True:
                        ### enable autostart
                        self.prompt("/usr/sundtek/sun_dvb.sh autostart_t2")
                else:
                    ### dvb-s/ dvb-s2
                    if config.plugins.SundtekControlCenter.sunconf.autostart.value == True:
                        ### enable autostart
                        self.prompt("/usr/sundtek/sun_dvb.sh autostart_s2")
        return True

    def tunerstart(self):
        for x in self["config"].list:
            x[1].save()
        configfile.save()
        self.sundtekconfigfile()
        self.setsettings()
        if (os.path.exists("/opt/bin/mediasrv")) and (os.path.exists("/opt/bin/mediaclient")) and (os.path.exists("/usr/sundtek/sun_dvb.sh")):
            try:
                from enigma import eEPGCache
                epgcache = eEPGCache.getInstance()
                epgcache.save()
            except:
                pass
            if config.plugins.SundtekControlCenter.dvbtransmission.value == "0":
                ### dvb-s/ dvb-s2
                self.prompt("/usr/sundtek/sun_dvb.sh start_s2")
            elif config.plugins.SundtekControlCenter.dvbtransmission.value == "1":
                ### dvb-c
                self.prompt("/usr/sundtek/sun_dvb.sh start_c")
            elif config.plugins.SundtekControlCenter.dvbtransmission.value == "2":
                ### dvb-t
                self.prompt("/usr/sundtek/sun_dvb.sh start_t")
            else:
                ### dvb-t2
                self.prompt("/usr/sundtek/sun_dvb.sh start_t2")
            if config.plugins.SundtekControlCenter.usbnet.selection.value == "1":
                ### networking
                if config.plugins.SundtekControlCenter.dvbtransmission.value == "0": #start DVBS
                    self.prompt("/usr/sundtek/sun_dvb.sh start_netdvb")
                else:
                    self.prompt("/usr/sundtek/sun_dvb.sh start_net")
